"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
var composeRequest = function composeRequest(ajv, escapeUrl, resolveDataForInspection, resolveDataForTransmission, resolveUrl, uriTemplates) {
    var ajvRemoveAdditional = ajv({ removeAdditional: true });

    return function (requestDef) {
        return function (link) {
            var context = requestDef.context || {};
            var uriTemplate = uriTemplates(escapeUrl(link.href));
            var missingKeys = uriTemplate.varNames.filter(function (varName) {
                return varName !== "%73elf";
            }).map(function (varName) {
                return varName === "%65mpty" ? "" : varName;
            }).filter(function (varName) {
                return !context.hasOwnProperty(varName);
            });

            if (missingKeys.length) {
                throw new Error("Missing keys for request: " + missingKeys.join(", "));
            }

            var request = {
                method: link.method || "GET",
                headers: {
                    "Accept": "application/json",
                    "Content-Type": "application/json"
                }
            };

            if (link.schema) {
                var isOk = ajvRemoveAdditional.compile(link.schema);
                if (!isOk(resolveDataForInspection(requestDef.data))) {
                    throw new Error(isOk.errors.map(function (err) {
                        return err.dataPath + " " + err.message;
                    }).join("; "));
                } else {
                    request.body = resolveDataForTransmission(requestDef.data);
                }
            }

            var url = resolveUrl(uriTemplate.fill(Object.assign({ "%73elf": context }, context)), requestDef);

            return {
                url: url,
                request: request,
                link: link
            };
        };
    };
};

var injection = exports.injection = function injection(container) {
    return container.mapClass("composeRequest", composeRequest, true);
};

exports.default = composeRequest;